"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@hibas123/utils");
const consolewriter_1 = require("./consolewriter");
const inspect_1 = require("./inspect");
const types_1 = require("./types");
const _1 = require(".");
const browser = typeof window !== "undefined";
function removeColors(text) {
    text = text.replace(/[\u001b\u009b][[()#;?]*(?:[0-9]{1,4}(?:;[0-9]{0,4})*)?[0-9A-ORZcf-nqry=><]/g, "");
    // let index = text.indexOf("\x1b");
    // while (index >= 0) {
    //     text = text.substring(0, index) + text.substring(index + 5, text.length);
    //     index = text.indexOf("\x1b");
    // }
    return text;
}
exports.removeColors = removeColors;
class LoggingBase {
    constructor(options) {
        this._formatMap = new types_1.DefaultFormatConfig();
        this.adapter = new Set();
        this.adapter_init = [];
        this.timerMap = new Map();
        this.messageObservable = new utils_1.Observable();
        this._logLevel = types_1.LoggingTypes.Debug;
        let opt;
        if (!options)
            opt = {};
        else if (typeof options === "string") {
            opt = { name: options };
        }
        else {
            opt = options;
        }
        let config = Object.assign({ name: undefined, console: true }, opt);
        if (config.name)
            this._name = config.name;
        for (let key in this) {
            if (typeof this[key] === "function")
                this[key] = this[key].bind(this);
        }
        if (config.console) {
            this.addAdapter(new consolewriter_1.ConsoleAdapter());
        }
        //Binding function to this
        this.debug = this.debug.bind(this);
        this.log = this.log.bind(this);
        this.warn = this.warn.bind(this);
        this.warning = this.warning.bind(this);
        this.error = this.error.bind(this);
        this.errorMessage = this.errorMessage.bind(this);
        this.flush = this.flush.bind(this);
    }
    set formatMap(value) {
        this._formatMap = value;
    }
    get logLevel() {
        return this._logLevel;
    }
    set logLevel(value) {
        this._logLevel = value;
    }
    get name() {
        return this._name;
    }
    addAdapter(adapter) {
        if (!this.adapter.has(adapter)) {
            this.adapter.add(adapter);
            let prms = Promise.resolve(adapter.init(this.messageObservable.getPublicApi(), this._name));
            this.adapter_init.push(prms);
        }
    }
    flush(sync) {
        if (sync) {
            this.adapter.forEach((elm) => elm.flush(true));
        }
        else {
            let adapters = [];
            this.adapter.forEach((elm) => adapters.push(elm.flush(false)));
            return Promise.all(adapters).then(() => { });
        }
    }
    close() {
        this.adapter.forEach((adapter) => adapter.close ? adapter.close() : undefined);
    }
    waitForSetup() {
        return Promise.all(this.adapter_init);
    }
    debug(...message) {
        if (this._logLevel <= types_1.LoggingTypes.Debug)
            this.message(types_1.LoggingTypes.Debug, message);
    }
    log(...message) {
        if (this._logLevel <= types_1.LoggingTypes.Log)
            this.message(types_1.LoggingTypes.Log, message);
    }
    warning(...message) {
        if (this._logLevel <= types_1.LoggingTypes.Warning)
            this.message(types_1.LoggingTypes.Warning, message);
    }
    warn(...message) {
        if (this._logLevel <= types_1.LoggingTypes.Warning)
            this.message(types_1.LoggingTypes.Warning, message);
    }
    error(error) {
        if (this._logLevel > types_1.LoggingTypes.Error)
            return;
        if (!error)
            error = "Empty ERROR was passed, so no informations available";
        if (typeof error === "string") {
            let e = new Error("This is a fake error, to get a stack trace");
            this.message(types_1.LoggingTypes.Error, [error, "\n", e.stack]);
        }
        else {
            this.message(types_1.LoggingTypes.Error, [error.message, "\n", error.stack], getCallerFromExisting(error));
        }
    }
    errorMessage(...message) {
        if (this._logLevel <= types_1.LoggingTypes.Error)
            this.message(types_1.LoggingTypes.Error, message);
    }
    getCurrentTime() {
        if (browser && window.performance && window.performance.now) {
            return window.performance.now();
        }
        else {
            return Date.now();
        }
    }
    /**
     * The time difference in milliseconds (fractions allowed!)
     * @param start Start time from getCurrentTime
     */
    getTimeDiff(start) {
        if (browser && window.performance && window.performance.now) {
            return window.performance.now() - start;
        }
        else {
            return Date.now() - start;
        }
    }
    time(id, name = id) {
        if (!id) {
            id = Math.floor(Math.random() * 899999 + 100000).toString();
        }
        this.timerMap.set(id, {
            name,
            start: this.getCurrentTime(),
        });
        return {
            id,
            end: () => this.timeEnd(id),
        };
    }
    timeEnd(id) {
        let timer = this.timerMap.get(id);
        if (timer) {
            let diff = this.getTimeDiff(timer.start);
            _1.default.message(types_1.LoggingTypes.Debug, [
                withColor(types_1.Colors.GREEN, `[${timer.name}]`),
                `->`,
                withColor(types_1.Colors.BLUE, diff.toFixed(4)),
                "ms",
            ]);
        }
    }
    message(type, message, caller) {
        let date = new Date().toISOString().replace(/T/, " ").replace(/\..+/, "");
        let file_raw = caller || getCallerFile();
        let file = `${file_raw.file}:${String(file_raw.line).padEnd(3, " ")}`;
        let type_str = types_1.LoggingTypes[type].toUpperCase().padEnd(5, " ");
        let type_format = [];
        switch (type) {
            case types_1.LoggingTypes.Log:
                type_format = this._formatMap.log;
                break;
            case types_1.LoggingTypes.Error:
                type_format = this._formatMap.error;
                break;
            case types_1.LoggingTypes.Debug:
                type_format = this._formatMap.debug;
                break;
            case types_1.LoggingTypes.Warning:
                type_format = this._formatMap.warning;
                break;
        }
        const prefix = [];
        const a = (text, formats = []) => {
            prefix.push({
                text,
                formats,
            });
        };
        a("[");
        a(date, this._formatMap.date);
        a("][");
        a(type_str, type_format);
        if (file_raw.file) {
            a("][");
            a(file, this._formatMap.file);
        }
        a("]: ");
        let raw = [];
        const formatted = [];
        let line;
        const newLine = () => {
            if (line && line.length > 0) {
                formatted.push(line);
                raw.push(line.map((e) => e.text).join(""));
            }
            line = [...prefix];
        };
        newLine();
        message.forEach((e, i) => {
            let formats = [];
            if (typeof e !== "string") {
                if (e && typeof e === "object") {
                    if (e[colorSymbol]) {
                        formats.push({
                            type: types_1.FormatTypes.COLOR,
                            color: e[colorSymbol],
                        });
                        e = e.value;
                    }
                }
                if (typeof e !== "string")
                    e = inspect_1.default(e, {
                        colors: true,
                        showHidden: true,
                        depth: 3,
                    });
            }
            removeColors(e)
                .split("\n")
                .map((text, index, { length }) => {
                line.push({ text, formats });
                if (index < length - 1) {
                    newLine();
                }
            });
            if (!e.endsWith("\n") && i < message.length - 1) {
                line.push({ text: " ", formats: [] });
            }
        });
        newLine();
        let msg = {
            date: new Date(),
            file,
            name: this._name,
            text: {
                raw,
                formatted,
            },
            type,
        };
        this.messageObservable.send(msg);
    }
}
exports.LoggingBase = LoggingBase;
const colorSymbol = Symbol("color");
function withColor(color, value) {
    return {
        [colorSymbol]: color,
        value,
    };
}
exports.withColor = withColor;
function getStack() {
    // Save original Error.prepareStackTrace
    let origPrepareStackTrace = Error.prepareStackTrace;
    // Override with function that just returns `stack`
    Error.prepareStackTrace = function (_, stack) {
        return stack;
    };
    // Create a new `Error`, which automatically gets `stack`
    let err = new Error();
    // Evaluate `err.stack`, which calls our new `Error.prepareStackTrace`
    let stack = err.stack;
    // Restore original `Error.prepareStackTrace`
    Error.prepareStackTrace = origPrepareStackTrace;
    // Remove superfluous function call on stack
    stack.shift(); // getStack --> Error
    return stack;
}
function baseName(path) {
    return path.split(/[\\/]/).pop();
}
function getCallerFile() {
    try {
        let stack = getStack();
        let current_file = stack.shift().getFileName();
        while (stack.length) {
            let caller_file = stack.shift();
            if (current_file !== caller_file.getFileName())
                return {
                    file: baseName(caller_file.getFileName()),
                    line: caller_file.getLineNumber(),
                };
        }
    }
    catch (err) { }
    return { file: undefined, line: 0 };
}
function getCallerFromExisting(err) {
    if (!err || !err.stack)
        return { file: "NOFILE", line: 0 };
    let lines = err.stack.split("\n");
    lines.shift(); // removing first line
    while (lines.length > 0) {
        let line = lines.shift();
        let matches = line.match(/[a-zA-Z_-]+[.][a-zA-Z_-]+[:][0-9]+/g);
        if (matches && matches.length > 0) {
            let [f, line] = matches[0].split(":");
            return {
                file: f,
                line: Number(line),
            };
        }
    }
}
//# sourceMappingURL=base.js.map